### *12FF MODEL WITH POSTERIOR PREDICTIVE DISTRIBUTIONS FOR ALL VARIABLES

model {
	for (j in posteriorStartMonth:posteriorEndMonth){  
	
		########################################## 
		## Priors as described in section 2.2.2
		## Data are fed in through R in jags.model
		## function call, via data parameter
		## Note: distribution parameters are 
		## mean and precision
		## precision = 1/variance
		##########################################
		
		### SUPERIOR
		superiorPrecip[j] ~ dgamma(superiorPriorPrecipShape[m[j]], superiorPriorPrecipRate[m[j]])
		superiorEvap[j] ~ dnorm(superiorEvapPriorMean[m[j]], superiorEvapPriorPrecision[m[j]])
		superiorRunoff[j] 	<- exp(superiorLogRunoff[j])
		superiorLogRunoff[j] ~ dnorm(superiorRunoffLogPriorMean[m[j]], superiorRunoffLogPriorPrecision[m[j]])	
		superiorOutflow[j] ~ dnorm(superiorOutflowPriorMean[m[j]], superiorOutflowPriorPrecision[m[j]])	
		superiorDiversion[j] ~ dnorm(superiorDiversionPriorMean[m[j]], superiorDiversionPriorPrecision[m[j]])	

		### MICHIGAN-HURON
		miHuronPrecip[j]	~ dgamma(miHuronPriorPrecipShape[m[j]], miHuronPriorPrecipRate[m[j]])			    
		miHuronEvap[j] ~ dnorm(miHuronEvapPriorMean[m[j]], miHuronEvapPriorPrecision[m[j]])		    
		miHuronRunoff[j] 	<- exp(miHuronLogRunoff[j])                            
		miHuronLogRunoff[j] ~ dnorm(miHuronRunoffLogPriorMean[m[j]], miHuronRunoffLogPriorPrecision[m[j]])
		miHuronOutflow[j] ~ dnorm(miHuronOutflowPriorMean[m[j]], miHuronOutflowPriorPrecision[m[j]]) 
		miHuronDiversion[j] ~ dnorm(miHuronDiversionPriorMean[m[j]], miHuronDiversionPriorPrecision[m[j]]) 
		
		########################################################
		## Likelihood functions as described in section 2.2.2
		## Biases follow equation 6
		########################################################
		
		### SUPERIOR
		ySuperiorPrecip1[j] ~ dnorm(ySuperiorPrecip1Mean[j], ySuperiorPrecip1Prec)						
		ySuperiorPrecip2[j] ~ dnorm(ySuperiorPrecip2Mean[j], ySuperiorPrecip2Prec)
		ySuperiorEvap1[j] ~ dnorm(ySuperiorEvap1Mean[j], ySuperiorEvap1Prec)
		ySuperiorEvap2[j] ~ dnorm(ySuperiorEvap2Mean[j], ySuperiorEvap2Prec)
		ySuperiorRunoff1[j] ~ dnorm(ySuperiorRunoff1Mean[j], ySuperiorRunoff1Prec)
		ySuperiorRunoff2[j] ~ dnorm(ySuperiorRunoff2Mean[j], ySuperiorRunoff2Prec)
		ySuperiorOutflow1[j]  ~ dnorm(ySuperiorOutflow1Mean[j], ySuperiorOutflow1Prec)
		ySuperiorOutflow2[j]  ~ dnorm(ySuperiorOutflow2Mean[j], ySuperiorOutflow2Prec)
		ySuperiorDiversion1[j]  ~ dnorm(ySuperiorDiversion1Mean[j], ySuperiorDiversion1Prec)

		ySuperiorPrecip1Mean[j] <- superiorPrecip[j] + ySuperiorPrecip1Bias[m[j]]
		ySuperiorPrecip2Mean[j] <- superiorPrecip[j] + ySuperiorPrecip2Bias[m[j]]
		ySuperiorEvap1Mean[j] <- superiorEvap[j] + ySuperiorEvap1Bias[m[j]]
		ySuperiorEvap2Mean[j] <- superiorEvap[j] + ySuperiorEvap2Bias[m[j]]
		ySuperiorRunoff1Mean[j] <- superiorRunoff[j] + ySuperiorRunoff1Bias[m[j]]	
		ySuperiorRunoff2Mean[j] <- superiorRunoff[j] + ySuperiorRunoff2Bias[m[j]]	
		ySuperiorOutflow1Mean[j] <- superiorOutflow[j] + ySuperiorOutflow1Bias[m[j]]
		ySuperiorOutflow2Mean[j] <- superiorOutflow[j] + ySuperiorOutflow2Bias[m[j]]
		ySuperiorDiversion1Mean[j] <- superiorDiversion[j] + ySuperiorDiversion1Bias[m[j]]

		### MICHIGAN-HURON
		yMiHuronPrecip1[j] ~ dnorm(yMiHuronPrecip1Mean[j], yMiHuronPrecip1Prec)						
		yMiHuronPrecip2[j] ~ dnorm(yMiHuronPrecip2Mean[j], yMiHuronPrecip2Prec)
		yMiHuronEvap1[j] ~ dnorm(yMiHuronEvap1Mean[j], yMiHuronEvap1Prec)
		yMiHuronEvap2[j] ~ dnorm(yMiHuronEvap2Mean[j], yMiHuronEvap2Prec)
		yMiHuronRunoff1[j] ~ dnorm(yMiHuronRunoff1Mean[j], yMiHuronRunoff1Prec)
		yMiHuronRunoff2[j] ~ dnorm(yMiHuronRunoff2Mean[j], yMiHuronRunoff2Prec)
		yMiHuronOutflow1[j]  ~ dnorm(yMiHuronOutflow1Mean[j], yMiHuronOutflow1Prec)
		yMiHuronOutflow2[j]  ~ dnorm(yMiHuronOutflow2Mean[j], yMiHuronOutflow2Prec)
		yMiHuronDiversion1[j]  ~ dnorm(yMiHuronDiversion1Mean[j], yMiHuronDiversion1Prec)

		yMiHuronPrecip1Mean[j] <- miHuronPrecip[j] + yMiHuronPrecip1Bias[m[j]]
		yMiHuronPrecip2Mean[j] <- miHuronPrecip[j] + yMiHuronPrecip2Bias[m[j]]
		yMiHuronEvap1Mean[j] <- miHuronEvap[j] + yMiHuronEvap1Bias[m[j]]
		yMiHuronEvap2Mean[j] <- miHuronEvap[j] + yMiHuronEvap2Bias[m[j]]
		yMiHuronRunoff1Mean[j] <- miHuronRunoff[j] + yMiHuronRunoff1Bias[m[j]]	
		yMiHuronRunoff2Mean[j] <- miHuronRunoff[j] + yMiHuronRunoff2Bias[m[j]]	
		yMiHuronOutflow1Mean[j] <- miHuronOutflow[j] + yMiHuronOutflow1Bias[m[j]]
		yMiHuronOutflow2Mean[j] <- miHuronOutflow[j] + yMiHuronOutflow2Bias[m[j]]
		yMiHuronDiversion1Mean[j] <- miHuronDiversion[j] + yMiHuronDiversion1Bias[m[j]]
		
	} 
	
	#####################################################
	# Water balance model as described in section 2.1
	# Model/Process error follows equation 4
	#####################################################
	
	for(k in rollPeriod:posteriorEndMonth){
		### SUPERIOR
		ySuperiorRStore[k] ~ dnorm(superiorRStore[k], ySuperiorRStorePrec)    	
		
		superiorRStore[k] <- (
			sum(superiorPrecip[(k-rollPeriod+1):k])
			-sum(superiorEvap[(k-rollPeriod+1):k])
			+sum(superiorRunoff[(k-rollPeriod+1):k])
			-sum(superiorOutflow[(k-rollPeriod+1):k])
			+sum(superiorDiversion[(k-rollPeriod+1):k])
			+sum(superiorProcError[m[(k-rollPeriod+1):k]])
		)
		
		### MICHIGAN-HURON
		yMiHuronRStore[k] ~ dnorm(miHuronRStore[k], yMiHuronRStorePrec) 
		
		miHuronRStore[k] <- (
			sum(miHuronPrecip[(k-rollPeriod+1):k])
			-sum(miHuronEvap[(k-rollPeriod+1):k])
			+sum(miHuronRunoff[(k-rollPeriod+1):k])
			-sum(miHuronOutflow[(k-rollPeriod+1):k])
			+0.70110653739*sum(superiorOutflow[(k-rollPeriod+1):k])
			-sum(miHuronDiversion[(k-rollPeriod+1):k])
			+sum(miHuronProcError[m[(k-rollPeriod+1):k]])
		)
	}
	
				   
	####################################################################
	## Bias terms and process errors, equations 4 and 6 respectively
	## Includes incorporation of expert guidance on Q and D
	## They would otherwise have the same bias definition as the other 
	## variables	
	####################################################################
	for (i in 1:12){
		ySuperiorPrecip1Bias[i] ~ dnorm(0,0.01)
		ySuperiorPrecip2Bias[i] ~ dnorm(0,0.01)
		ySuperiorEvap1Bias[i] ~ dnorm(0,0.01)
		ySuperiorEvap2Bias[i] ~ dnorm(0,0.01)
		ySuperiorRunoff1Bias[i] ~ dnorm(0,0.01)
		ySuperiorRunoff2Bias[i] ~ dnorm(0,0.01)
		ySuperiorOutflow1Bias[i] ~ dnorm(0,0.25)
		ySuperiorOutflow2Bias[i] ~ dnorm(0,0.25)
		ySuperiorDiversion1Bias[i] ~ dnorm(0,0.25)
		
		yMiHuronPrecip1Bias[i] ~ dnorm(0,0.01)
		yMiHuronPrecip2Bias[i] ~ dnorm(0,0.01)
		yMiHuronEvap1Bias[i] ~ dnorm(0,0.01)
		yMiHuronEvap2Bias[i] ~ dnorm(0,0.01)
		yMiHuronRunoff1Bias[i] ~ dnorm(0,0.01)
		yMiHuronRunoff2Bias[i] ~ dnorm(0,0.01)
		yMiHuronOutflow1Bias[i] ~ dnorm(0,0.25)
		yMiHuronOutflow2Bias[i] ~ dnorm(0,0.25)
		yMiHuronDiversion1Bias[i] ~ dnorm(0,0.25)
		
		superiorProcError[i] ~ dnorm(0,0.01)
		miHuronProcError[i] ~ dnorm(0,0.01)
	}
	
	##############################
	## Precision for Observations
	##############################
	
	### SUPERIOR 
	ySuperiorRStorePrec	  ~ dgamma(0.01,0.01)    	
	ySuperiorPrecip1Prec  ~ dgamma(0.1,0.1)      
	ySuperiorPrecip2Prec  ~ dgamma(0.1,0.1)      
	ySuperiorEvap1Prec    ~ dgamma(0.1,0.1)      
	ySuperiorEvap2Prec    ~ dgamma(0.1,0.1)      
	ySuperiorRunoff1Prec  ~ dgamma(0.1,0.1)
	ySuperiorRunoff2Prec  ~ dgamma(0.1,0.1)
	ySuperiorOutflow1Prec ~ dgamma(0.1,0.1)      
	ySuperiorOutflow2Prec ~ dgamma(0.1,0.1)
	ySuperiorDiversion1Prec ~ dgamma(0.1,0.1)
	
	### MICHIGAN-HURON							 
	yMiHuronRStorePrec ~ dgamma(0.01,0.01)    
	yMiHuronPrecip1Prec ~ dgamma(0.1,0.1)      
	yMiHuronPrecip2Prec ~ dgamma(0.1,0.1)      
	yMiHuronEvap1Prec ~ dgamma(0.1,0.1)      
	yMiHuronEvap2Prec ~ dgamma(0.1,0.1)      
	yMiHuronRunoff1Prec ~ dgamma(0.1,0.1) 
	yMiHuronRunoff2Prec ~ dgamma(0.1,0.1) 
	yMiHuronOutflow1Prec ~ dgamma(0.1,0.1)      
	yMiHuronOutflow2Prec ~ dgamma(0.1,0.1)
	yMiHuronDiversion1Prec ~ dgamma(0.1,0.1)
	
	
	################################################################
	# Posterior predictive (PP) distributions for verification	 
	################################################################
	
	for(jp in posteriorStartMonth:posteriorEndMonth){		
		### SUPERIOR
		ySuperiorPrecip1PP[jp] ~ dnorm(ySuperiorPrecip1Mean[jp], ySuperiorPrecip1Prec)						
		ySuperiorPrecip2PP[jp] ~ dnorm(ySuperiorPrecip2Mean[jp], ySuperiorPrecip2Prec)
		ySuperiorEvap1PP[jp] ~ dnorm(ySuperiorEvap1Mean[jp], ySuperiorEvap1Prec)
		ySuperiorEvap2PP[jp] ~ dnorm(ySuperiorEvap2Mean[jp], ySuperiorEvap2Prec)
		ySuperiorRunoff1PP[jp] ~ dnorm(ySuperiorRunoff1Mean[jp], ySuperiorRunoff1Prec)
		ySuperiorRunoff2PP[jp] ~ dnorm(ySuperiorRunoff2Mean[jp], ySuperiorRunoff2Prec)
		ySuperiorOutflow1PP[jp] ~ dnorm(ySuperiorOutflow1Mean[jp], ySuperiorOutflow1Prec)
		ySuperiorOutflow2PP[jp] ~ dnorm(ySuperiorOutflow2Mean[jp], ySuperiorOutflow2Prec)
		ySuperiorDiversion1PP[jp] ~ dnorm(ySuperiorDiversion1Mean[jp], ySuperiorDiversion1Prec)

		### MICHIGAN-HURON
		yMiHuronPrecip1PP[jp] ~ dnorm(yMiHuronPrecip1Mean[jp], yMiHuronPrecip1Prec)						
		yMiHuronPrecip2PP[jp] ~ dnorm(yMiHuronPrecip2Mean[jp], yMiHuronPrecip2Prec)
		yMiHuronEvap1PP[jp] ~ dnorm(yMiHuronEvap1Mean[jp], yMiHuronEvap1Prec)
		yMiHuronEvap2PP[jp] ~ dnorm(yMiHuronEvap2Mean[jp], yMiHuronEvap2Prec)
		yMiHuronRunoff1PP[jp] ~ dnorm(yMiHuronRunoff1Mean[jp], yMiHuronRunoff1Prec)
		yMiHuronRunoff2PP[jp] ~ dnorm(yMiHuronRunoff2Mean[jp], yMiHuronRunoff2Prec)
		yMiHuronOutflow1PP[jp] ~ dnorm(yMiHuronOutflow1Mean[jp], yMiHuronOutflow1Prec)
		yMiHuronOutflow2PP[jp] ~ dnorm(yMiHuronOutflow2Mean[jp], yMiHuronOutflow2Prec)
		yMiHuronDiversion1PP[jp] ~ dnorm(yMiHuronDiversion1Mean[jp], yMiHuronDiversion1Prec)
		
		# MONTH BY MONTH CHANGE IN STORAGE ANALYSIS
		
		ySuperiorDStorePP[jp] ~ dnorm(superiorDStore[jp], ySuperiorRStorePrec)
		
		superiorDStore[jp] <- (
			superiorPrecip[jp]
			-superiorEvap[jp]
			+superiorRunoff[jp]
			-superiorOutflow[jp]
			+superiorDiversion[jp]
			+superiorProcError[m[jp]]
		)
		
		yMiHuronDStorePP[jp] ~ dnorm(miHuronDStore[jp], yMiHuronRStorePrec)
		
		miHuronDStore[jp] <- (
			miHuronPrecip[jp]
			-miHuronEvap[jp]
			+miHuronRunoff[jp]
			+0.70110653739*superiorOutflow[jp]
			-miHuronOutflow[jp]
			-miHuronDiversion[jp]
			+miHuronProcError[m[jp]]
		)
	}
	
	### ROLLING SUM STORAGE ANALYSIS
	
	# 1 YEAR
	
	for(x in 12:posteriorEndMonth){
		ySuperiorR1YStorePP[x] ~ dnorm(superiorR1YStore[x], ySuperiorRStorePrec)
		
		superiorR1YStore[x] <- (
			sum(superiorPrecip[(x-12+1):x])
			-sum(superiorEvap[(x-12+1):x])
			+sum(superiorRunoff[(x-12+1):x])
			-sum(superiorOutflow[(x-12+1):x])
			+sum(superiorDiversion[(x-12+1):x])
			+sum(superiorProcError[m[(x-12+1):x]])
		)
		
		yMiHuronR1YStorePP[x] ~ dnorm(miHuronR1YStore[x], yMiHuronRStorePrec)
		
		miHuronR1YStore[x] <- (
			sum(miHuronPrecip[(x-12+1):x])
			-sum(miHuronEvap[(x-12+1):x])
			+sum(miHuronRunoff[(x-12+1):x])
			+0.70110653739*sum(superiorOutflow[(x-12+1):x])
			-sum(miHuronOutflow[(x-12+1):x])
			-sum(miHuronDiversion[(x-12+1):x])
			+sum(miHuronProcError[m[(x-12+1):x]])
		)
	}	
	
	# 5 YEAR
	
	for(z in 60:posteriorEndMonth){
		ySuperiorR5YStorePP[z] ~ dnorm(superiorR5YStore[z], ySuperiorRStorePrec)
		
		superiorR5YStore[z] <- (
			sum(superiorPrecip[(z-60+1):z])
			-sum(superiorEvap[(z-60+1):z])
			+sum(superiorRunoff[(z-60+1):z])
			-sum(superiorOutflow[(z-60+1):z])
			+sum(superiorDiversion[(z-60+1):z])
			+sum(superiorProcError[m[(z-60+1):z]])
		)
		
		yMiHuronR5YStorePP[z] ~ dnorm(miHuronR5YStore[z], yMiHuronRStorePrec)
		
		miHuronR5YStore[z] <- (
			sum(miHuronPrecip[(z-60+1):z])
			-sum(miHuronEvap[(z-60+1):z])
			+sum(miHuronRunoff[(z-60+1):z])
			+0.70110653739*sum(superiorOutflow[(z-60+1):z])
			-sum(miHuronOutflow[(z-60+1):z])
			-sum(miHuronDiversion[(z-60+1):z])
			+sum(miHuronProcError[m[(z-60+1):z]])
		)
	}

}




